#!/usr/bin/python3
###############################################################################
#                                                                             #
# location-database - A database to determine someone's                       #
#                     location on the Internet                                #
# Copyright (C) 2018 Michael Tremer                                           #
#                                                                             #
# This program is free software: you can redistribute it and/or modify        #
# it under the terms of the GNU General Public License as published by        #
# the Free Software Foundation, either version 3 of the License, or           #
# (at your option) any later version.                                         #
#                                                                             #
# This program is distributed in the hope that it will be useful,             #
# but WITHOUT ANY WARRANTY; without even the implied warranty of              #
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the               #
# GNU General Public License for more details.                                #
#                                                                             #
# You should have received a copy of the GNU General Public License           #
# along with this program.  If not, see <http://www.gnu.org/licenses/>.       #
#                                                                             #
###############################################################################

import iso3166
import logging
import re

import location

from . import util

RE_AS = re.compile("^AS(\d+)$")

class Database(object):
	def __init__(self, vendor, description=None, license=None):
		self.writer = location.Writer()

		# Set metadata
		if vendor:
			self.writer.vendor = vendor

		if description:
			self.writer.description = description

		if license:
			self.writer.license = license

	def write(self, path):
		self.writer.write(path)

	def import_ases(self, filename):
		with open(filename, "rb") as f:
			for block in util.iterate_over_blocks(f):
				self._parse_asnum_block(block)

	def import_networks(self, filename):
		with open(filename, "rb") as f:
			for block in util.iterate_over_blocks(f):
				self._parse_network_block(block)

	def import_rir(self, rir):
		self.import_ases(rir.filename_asnums)
		self.import_networks(rir.filename_networks)

	def _parse_asnum_block(self, block):
		asn  = None
		name = None

		for line in block:
			key, val = util.split_line(line)

			if key == "asnum":
				m = RE_AS.match(val)
				if m:
					asn = int(m.group(1))

			elif key == "org":
				name = val

		if asn and name:
			a = self.writer.add_as(asn)
			a.name = name

			logging.debug("Added new AS: %s" % a)

	def _parse_network_block(self, block):
		network = None
		asn     = None
		country = None

		for line in block:
			key, val = util.split_line(line)

			if key == "net":
				network = val

			elif key == "asnum":
				m = RE_AS.match(val)
				if m:
					asn = int(m.group(1))

			elif key == "country":
				country = val

		if network and country:
			try:
				n = self.writer.add_network(network)
			except IndexError:
				logging.warning("Skipping network %s, because one already exists at this address" % network)
				return

			# Translate some special country codes into flags
			if country == "A1":
				n.set_flag(location.NETWORK_FLAG_ANONYMOUS_PROXY)
			elif country == "A2":
				n.set_flag(location.NETWORK_FLAG_SATELLITE_PROVIDER)
			elif country == "A3":
				n.set_flag(location.NETWORK_FLAG_ANYCAST)

			# Set all other country codes
			else:
				n.country_code = country

			# Set ASN if available
			if asn:
				n.asn = asn

			logging.debug("Added new network: %s" % n)

	def import_countries(self, filename):
		with open(filename) as f:
			for line in f:
				if line.startswith("#"):
					continue

				try:
					country_code, continent_code = line.split()
				except:
					logging.error("Could not parse line in %s: %s" (filename, line))
					raise

				c = self.writer.add_country(country_code)

				# Add continent code
				if continent_code:
					c.continent_code = continent_code

				# Add name
				country = iso3166.countries_by_alpha2.get(country_code)
				if country:
					c.name = country.apolitical_name
